import 'package:packme/packme.dart';

class Animal extends PackMeMessage {
	Animal({
		required this.hp,
	});
	Animal.$empty();

	late double hp;

	@override
	int $estimate() {
		$reset();
		return 8;
	}

	@override
	void $pack() {
		$packDouble(hp);
	}

	@override
	void $unpack() {
		hp = $unpackDouble();
	}

	@override
	String toString() {
		return 'Animal\x1b[0m(hp: ${PackMe.dye(hp)})';
	}
}

class AnimalMessage extends PackMeMessage {
	AnimalMessage({
		required this.hp,
	});
	AnimalMessage.$empty();

	late double hp;

	@override
	int $estimate() {
		$reset();
		return 8;
	}

	@override
	void $pack() {
		$packDouble(hp);
	}

	@override
	void $unpack() {
		hp = $unpackDouble();
	}

	@override
	String toString() {
		return 'AnimalMessage\x1b[0m(hp: ${PackMe.dye(hp)})';
	}
}

class Cat extends Animal {
	Cat({
		required double hp,
		required this.color,
	}) : super(hp: hp);
	Cat.$empty() : super.$empty();

	late String color;

	@override
	int $estimate() {
		int _bytes = super.$estimate();
		_bytes += $stringBytes(color);
		return _bytes;
	}

	@override
	void $pack() {
		super.$pack();
		$packString(color);
	}

	@override
	void $unpack() {
		super.$unpack();
		color = $unpackString();
	}

	@override
	String toString() {
		return 'Cat\x1b[0m(color: ${PackMe.dye(color)}) of ${super.toString()}';
	}
}

class TestMessageItem extends PackMeMessage {
	TestMessageItem({
		required this.a,
		required this.b,
		required this.c,
	});
	TestMessageItem.$empty();

	late int a;
	late int b;
	late int c;

	@override
	int $estimate() {
		$reset();
		return 3;
	}

	@override
	void $pack() {
		$packUint8(a);
		$packUint8(b);
		$packUint8(c);
	}

	@override
	void $unpack() {
		a = $unpackUint8();
		b = $unpackUint8();
		c = $unpackUint8();
	}

	@override
	String toString() {
		return 'TestMessageItem\x1b[0m(a: ${PackMe.dye(a)}, b: ${PackMe.dye(b)}, c: ${PackMe.dye(c)})';
	}
}

class TestMessageStats extends PackMeMessage {
	TestMessageStats({
		required this.a,
		required this.b,
		required this.c,
	});
	TestMessageStats.$empty();

	late int a;
	late int b;
	late int c;

	@override
	int $estimate() {
		$reset();
		return 3;
	}

	@override
	void $pack() {
		$packUint8(a);
		$packUint8(b);
		$packUint8(c);
	}

	@override
	void $unpack() {
		a = $unpackUint8();
		b = $unpackUint8();
		c = $unpackUint8();
	}

	@override
	String toString() {
		return 'TestMessageStats\x1b[0m(a: ${PackMe.dye(a)}, b: ${PackMe.dye(b)}, c: ${PackMe.dye(c)})';
	}
}

class TestMessage extends PackMeMessage {
	TestMessage({
		required this.stats,
		required this.items,
	});
	TestMessage.$empty();

	late TestMessageStats stats;
	late List<TestMessageItem> items;

	@override
	int $estimate() {
		$reset();
		int _bytes = 8;
		_bytes += stats.$estimate();
		_bytes += 4 + items.fold(0, (int a, TestMessageItem b) => a + b.$estimate());
		return _bytes;
	}

	@override
	void $pack() {
		$initPack(617692482);
		$packMessage(stats);
		$packUint32(items.length);
		for (int _i5 = 0; _i5 < items.length; _i5++) {
			$packMessage(items[_i5]);
		}
	}

	@override
	void $unpack() {
		$initUnpack();
		stats = $unpackMessage(TestMessageStats.$empty());
		items = List<TestMessageItem>.generate($unpackUint32(), (int i) {
			return $unpackMessage(TestMessageItem.$empty());
		});
	}

	@override
	String toString() {
		return 'TestMessage\x1b[0m(stats: ${PackMe.dye(stats)}, items: ${PackMe.dye(items)})';
	}
}

final Map<int, PackMeMessage Function()> p1MessageFactory = <int, PackMeMessage Function()>{
	617692482: () => TestMessage.$empty(),
};